#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <assert.h>

#include "werr.h"
#include "url_get.h"
#include "intl.h"

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_GHTTP_H
#  include <ghttp.h>
#else
/* No use having the library without the headers */
#  undef HAVE_LIBGHTTP
#endif

#define BUFSIZE 2048
#define TIMEOUT 30

#ifdef HAVE_LIBGHTTP
/* configure has found libghttp */

void
url_init_session(url_session *urls)
{
  /* Set sane starting values ready for next URL fetch */

  /*urls->url[0] = '\0' ;*/
  urls->session = -1 ;
  urls->request = NULL ;
  urls->status = 0  ;
  if (urls->body) free(urls->body) ;
  urls->body = NULL ;
  urls->count = 0 ;
  urls->body_alloc = 0 ;
  urls->idle = 0 ;
}

void
url_deregister(url_session *urls)
{
  /* Done with this URL */

  if (urls->request && (urls->session != -1))
    ghttp_request_destroy((ghttp_request *) urls->request) ;

  urls->request = NULL ;
  urls->session = -1 ;
}

int
url_failure(url_session *urls)
{
  url_deregister(urls) ;
  if (urls->body) 
  {
    free(urls->body) ;
    urls->body = NULL ;
  }
  return -1 ;
}

int
url_request(url_session *urls, char *url, char *proxy)
{
  ghttp_request *request ;
  
  if (!url) return -1 ;
  if (!url[0]) return -1 ;
    
  //assert(urls->request == NULL) ;

  url_init_session(urls) ;

  time(&urls->idle) ; /* Start idle timer */

  /* Allocate a new empty request object */

  request = ghttp_request_new() ;

  werr(WERR_DEBUG1, "Allocated http request") ;

  urls->request = request ;
  urls->session = 1 ; /* Historical - from RISC OS. But used here as a flag */

  if (proxy)
  {
    /* Enable the proxy */

    ghttp_set_proxy(request, proxy) ;
  }

  /* We need to work asynchronously */

  ghttp_set_sync(request, ghttp_async) ;

  /* Set the URL */

  werr(WERR_DEBUG0, "Fetch URL \"%s\"", url) ;

  ghttp_set_uri(request, url) ;

  /* Close the connexion after fetch */

  ghttp_set_header(request, http_hdr_Connection, "close") ;

  /* Prepare the connexion */

  ghttp_prepare(request) ;

  /* Send request and check initial status */

  urls->status = ghttp_process(request) ;

  if (urls->status == ghttp_error)
  {
    /* Request failed */
    werr(WERR_WARNING, "url_request Error: %s",
      ghttp_get_error(request)) ;

    url_failure(urls) ;
    return -1 /* Error */ ;
  }

  return urls->status ; /* OK - done or not_done */
}

void
url_stop(url_session *urls)
{
  /* Halt attempt to fetch URL - e.g. because it timed out */

  ghttp_close((ghttp_request *) urls->request) ;
}

int url_finished(url_session *urls)
{
  /* Has the fetch finished? */

  //return (urls->status != ghttp_not_done) ;
  return (urls->status == ghttp_done) ;
}

char *
url_get_body_part(url_session *urls)
{
  int len ;

  /* Try to retrieve the body of the URL fetch */

  /* Process request (again) if not done yet */

  if (urls->status == ghttp_not_done)
    urls->status = ghttp_process((ghttp_request *) urls->request) ;

  /* What is the status now? */

  switch (urls->status)
  {
    case ghttp_error:
      werr(WERR_WARNING, "Error fetching body: %s",
        ghttp_get_error((ghttp_request *) urls->request)) ;
      url_failure(urls) ;
      return NULL ;

    case ghttp_done:
      len = ghttp_get_body_len((ghttp_request *) urls->request) ;

      if (urls->body) free(urls->body) ;

      urls->body = (char *) malloc(len + 1) ;

      urls->body_alloc = len ;

      strncpy(urls->body,
        ghttp_get_body((ghttp_request *) urls->request), len) ;
      urls->body[len] = '\0' ;
      break ;

    case ghttp_not_done:
      if (urls->idle == 0) time(&urls->idle) ; /* Start idle timer */
  }

  return urls->body ;
}

/* Is there an active session? */

int
url_get_any_session(url_session *urls)
{
  return (urls->request != NULL) ;
}

#else
/* We do not have libghttp */

void
url_stop(url_session *urls)
{
  return ;
}

void
url_deregister(url_session *urls)
{
  return ;
}

void
url_init_session(url_session *urls)
{
  return ;
}

int
url_request(url_session *urls, char *url, char *proxy)
{
  werr(0, _("Oww was not compiled with libghttp.")) ;
  return -1 ;
}

int
url_failure(url_session *urls)
{
  return -1 ;
}

char *
url_get_body_part(url_session *urls)
{
  return NULL ;
}

int
url_finished(url_session *urls)
{
  return 1 ;
}

int
url_get_any_session(url_session *urls)
{
  return 0 ;
}

#endif
